
package MiniBuffer;
@ISA = qw( Fundamental );

# The mini buffer handles prompting and auto completion
# It takes a closure to be called once the prompting has completed

# It also keeps a history of past commands indexed by the prompt that was
# used.

####################################
# Class Variables

# Emacs state
foreach $key (values %main::KeyCode){
	$InputStateKeyMap{$key} = \&InsertChar;
}
$InputStateKeyMap{$main::KeyCode{"DEL"}}=\&MiniBuffer::DeleteCharOnLine;
$InputStateKeyMap{$main::KeyCode{"CR"}}=\&MiniBuffer::Select;
$InputStateKeyMap{$main::KeyCode{"KEY_BACKSPACE"}}=\&MiniBuffer::DeletePrevCharOnLine;
$InputStateKeyMap{$main::KeyCode{"KEY_LEFT"}}=\&MiniBuffer::MoveToPrevCharOnLine;
$InputStateKeyMap{$main::KeyCode{"KEY_RIGHT"}}=\&MiniBuffer::MoveToNextCharOnLine;
$InputStateKeyMap{$main::KeyCode{"KEY_HOME"}}=\&MiniBuffer::MoveToLineBeginning;
$InputStateKeyMap{$main::KeyCode{"KEY_END"}}=\&MiniBuffer::MoveToLineEnd;
$InputStateKeyMap{$main::KeyCode{"^L"}}=\&MiniBuffer::RedrawDisplay;
$InputStateKeyMap{$main::KeyCode{"^A"}}=\&MiniBuffer::MoveToLineBeginning;
$InputStateKeyMap{$main::KeyCode{"^E"}}=\&MiniBuffer::MoveToLineEnd;
$InputStateKeyMap{$main::KeyCode{"TAB"}}=\&MiniBuffer::Complete;

####################################
# Class Methods

sub new {
	#_DebugLog("Fund new\n");
	my($class,$prompt,$default,$closure) = @_;
	my($self) = {};
	bless $self, $class;
	$self->Initialize;

	$self->{"prompt"}=$self->Expand($prompt);
	${$self->{"buffer"}}[0]=$default;
	$self->{"closure"}=$closure;

	#&_DebugLog("MiniBuffer new $prompt $default $closure\n");
	$self->MoveToLineEnd;
	return $self;
}

####################################
# Instance Methods

sub Initialize {
	my($self) = shift;

	#_DebugLog("Fund Initialize\n");
	$self->{"trycol"} = $self->{"curcol"} = 0;
	$self->{"curline"} = 0;
	$self->{"buffer"} = [];
	$self->{"ro"} = 0;
	$self->{"modified"} = 0;
	&_MiniBuffer($self);
	return $self;
}

sub _MiniBuffer {
	my($self) = shift;
	$self->{"curdisp"} = []; 
        $self->{"mode desc"} = "MiniBuffer";
        $self->{"input state keymap"} = {%InputStateKeyMap};
        $self->ChangeToInputState;
}

sub DispXY {
	my($self) = shift;
	my($dispcol,$curline)= $self->SUPER::DispXY;
	return(length($self->{"prompt"})+$dispcol,$curline);
}

sub Display {
	my($self) = shift;
	my($curleft,$curtop,$wid,$hig) = @_;

	my($buffer) = $self->{"buffer"};
	my($curline) = $self->{"curline"};
	my($disp) = $self->{"prompt"}.$self->Expand($$buffer[$curline]);
	return("",substr($disp,$curleft,$wid));
}

sub EditViewChanged {
	my($self) = shift;

	$self->{'curcol'} = shift;
	$self->{'curline'} = shift;

	# we never want to return to a search in progress
	$self->{'incrementalSearch'} = 0;
}

sub Complete {
}

sub Select {
	my($self) = shift;

	#_DebugLog("MiniBuffer Select\n");
	my($buffer) = $self->{"buffer"};
        my($curline) = $self->{"curline"};
	#&_DebugLog("MiniBuffer Select executing ".$self->{"closure"}."\n");
	$self->Delete;
	&{$self->{"closure"}}($$buffer[$curline]);
}

###################
#Movement

sub MoveToPrevCharOnLine {
	my($self) = shift;
	return $self->SUPER::MoveToPrevCharOnLine(@_);
}

sub MoveToNextCharOnLine {
	my($self) = shift;
	return $self->SUPER::MoveToNextCharOnLine(@_);
}
sub MoveToLineBeginning {
	my($self) = shift;
	return $self->SUPER::MoveToLineBeginning(@_);
}
sub MoveToLineEnd {
	my($self) = shift;
	return $self->SUPER::MoveToLineEnd(@_);
}

###################
#Line Manipulation

sub DeleteToLineEnd {
	my($self) = shift;
	return $self->SUPER::DeleteToLineEnd(@_);
}
###################
#Character Manipulation

sub DeleteCharOnLine {
	my($self) = shift;
	return $self->SUPER::DeleteCharOnLine(@_);
}

sub DeletePrevCharOnLine {
	my($self) = shift;
	return $self->SUPER::DeletePrevCharOnLine(@_);
}

sub InsertChar {
	my($self) = shift;
	return $self->SUPER::InsertChar(@_);
}

###################
# Buffer Manipulation
# and Interface to main::

sub DebugLog { return _localize(\&_DebugLog,@_); }
sub _DebugLog { &main::DebugLog(@_); }

1;
